@@ -10,8 +10,9 @@
 #undef HAVE_MDB
 #undef HAVE_ORACLE
 #undef HAVE_PRIMEBASE
-#undef HAVE_TDS
+#undef HAVE_FREETDS
 #undef HAVE_SYBASE
+#undef HAVE_IBMDB2
 #undef SYBASE_DEBUG
 #undef HAVE_POSTGRES
 #undef HAVE_CAMEL
