@@ -26,6 +26,7 @@
 #include <libgda/gda-data-model-array.h>
 #include <libgda/gda-intl.h>
 #include <stdlib.h>
+#include <string.h>
 #include "gda-mysql.h"
 #include "gda-mysql-recordset.h"
 
@@ -38,6 +39,7 @@ static void gda_mysql_provider_init       (GdaMysqlProvider *provider,
 					   GdaMysqlProviderClass *klass);
 static void gda_mysql_provider_finalize   (GObject *object);
 
+static const gchar *gda_mysql_provider_get_version (GdaServerProvider *provider);
 static gboolean gda_mysql_provider_open_connection (GdaServerProvider *provider,
 						    GdaConnection *cnc,
 						    GdaQuarkList *params,
@@ -45,6 +47,13 @@ static gboolean gda_mysql_provider_open_connection (GdaServerProvider *provider,
 						    const gchar *password);
 static gboolean gda_mysql_provider_close_connection (GdaServerProvider *provider,
 						     GdaConnection *cnc);
+static const gchar *gda_mysql_provider_get_server_version (GdaServerProvider *provider,
+							   GdaConnection *cnc);
+static const gchar *gda_mysql_provider_get_database (GdaServerProvider *provider,
+						     GdaConnection *cnc);
+static gboolean gda_mysql_provider_change_database (GdaServerProvider *provider,
+		                                    GdaConnection *cnc,
+		                                    const gchar *name);
 static gboolean gda_mysql_provider_create_database (GdaServerProvider *provider,
 						    GdaConnection *cnc,
 						    const gchar *name);
@@ -57,13 +66,13 @@ static GList *gda_mysql_provider_execute_command (GdaServerProvider *provider,
 						  GdaParameterList *params);
 static gboolean gda_mysql_provider_begin_transaction (GdaServerProvider *provider,
 						      GdaConnection *cnc,
-						      const gchar *trans_id);
+						      GdaTransaction *xaction);
 static gboolean gda_mysql_provider_commit_transaction (GdaServerProvider *provider,
 						       GdaConnection *cnc,
-						       const gchar *trans_id);
+						       GdaTransaction *xaction);
 static gboolean gda_mysql_provider_rollback_transaction (GdaServerProvider *provider,
 							 GdaConnection *cnc,
-							 const gchar *trans_id);
+							 GdaTransaction *xaction);
 static gboolean gda_mysql_provider_supports (GdaServerProvider *provider,
 					     GdaConnection *cnc,
 					     GdaConnectionFeature feature);
@@ -87,8 +96,12 @@ gda_mysql_provider_class_init (GdaMysqlProviderClass *klass)
 	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->finalize = gda_mysql_provider_finalize;
+	provider_class->get_version = gda_mysql_provider_get_version;
 	provider_class->open_connection = gda_mysql_provider_open_connection;
 	provider_class->close_connection = gda_mysql_provider_close_connection;
+	provider_class->get_server_version = gda_mysql_provider_get_server_version;
+	provider_class->get_database = gda_mysql_provider_get_database;
+	provider_class->change_database = gda_mysql_provider_change_database;
 	provider_class->create_database = gda_mysql_provider_create_database;
 	provider_class->drop_database = gda_mysql_provider_drop_database;
 	provider_class->execute_command = gda_mysql_provider_execute_command;
@@ -146,6 +159,17 @@ gda_mysql_provider_new (void)
 	return GDA_SERVER_PROVIDER (provider);
 }
 
+/* get_version handler for the GdaMysqlProvider class */
+static const gchar *
+gda_mysql_provider_get_version (GdaServerProvider *provider)
+{
+	GdaMysqlProvider *myprv = (GdaMysqlProvider *) provider;
+
+	g_return_val_if_fail (GDA_IS_MYSQL_PROVIDER (myprv), NULL);
+
+	return VERSION;
+}
+
 /* open_connection handler for the GdaMysqlProvider class */
 static gboolean
 gda_mysql_provider_open_connection (GdaServerProvider *provider,
@@ -251,6 +275,25 @@ gda_mysql_provider_close_connection (GdaServerProvider *provider, GdaConnection
 	return TRUE;
 }
 
+/* get_server_version handler for the GdaMysqlProvider class */
+static const gchar *
+gda_mysql_provider_get_server_version (GdaServerProvider *provider, GdaConnection *cnc)
+{
+	MYSQL *mysql;
+	GdaMysqlProvider *myprv = (GdaMysqlProvider *) provider;
+
+	g_return_val_if_fail (GDA_IS_MYSQL_PROVIDER (myprv), NULL);
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+	
+	mysql = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_MYSQL_HANDLE);
+	if (!mysql) {
+		gda_connection_add_error_string (cnc, _("Invalid MySQL handle"));
+		return FALSE;
+	}
+
+	return (const gchar *) mysql->server_version;
+}
+
 static GList *
 process_sql_commands (GList *reclist, GdaConnection *cnc, const gchar *sql)
 {
@@ -296,6 +339,56 @@ process_sql_commands (GList *reclist, GdaConnection *cnc, const gchar *sql)
 	return reclist;
 }
 
+/* get_database handler for the GdaMysqlProvider class */
+static const gchar *
+gda_mysql_provider_get_database (GdaServerProvider *provider,
+				 GdaConnection *cnc)
+{
+	MYSQL *mysql;
+	GdaMysqlProvider *myprv = (GdaMysqlProvider *) provider;
+
+	g_return_val_if_fail (GDA_IS_MYSQL_PROVIDER (myprv), NULL);
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+
+	mysql = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_MYSQL_HANDLE);
+	if (!mysql) {
+		gda_connection_add_error_string (cnc, _("Invalid MYSQL handle"));
+		return NULL;
+	}
+
+	return (const gchar *) mysql->db;
+}
+
+/* change_database handler for the GdaMysqlProvider class */
+static gboolean
+gda_mysql_provider_change_database (GdaServerProvider *provider,
+		                    GdaConnection *cnc,
+				    const gchar *name)
+{
+	gint rc;
+	MYSQL *mysql;
+	GdaMysqlProvider *myprv = (GdaMysqlProvider *) provider;
+
+	g_return_val_if_fail (GDA_IS_MYSQL_PROVIDER (myprv), FALSE);
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
+	g_return_val_if_fail (name != NULL, FALSE);
+
+	mysql = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_MYSQL_HANDLE);
+	if (!mysql) {
+		gda_connection_add_error_string (cnc, _("Invalid MYSQL handle"));
+		return FALSE;
+	}
+
+	rc = mysql_select_db (mysql, name);
+	if (rc != 0) {
+		gda_connection_add_error (cnc, gda_mysql_make_error (mysql));
+		return FALSE;
+	}
+
+	return TRUE;
+
+}
+
 /* create_database handler for the GdaMysqlProvider class */
 static gboolean
 gda_mysql_provider_create_database (GdaServerProvider *provider,
@@ -303,8 +396,8 @@ gda_mysql_provider_create_database (GdaServerProvider *provider,
 				    const gchar *name)
 {
 	gint rc;
-	MYSQL *mysql;
 	gchar *sql;
+	MYSQL *mysql;
 	GdaMysqlProvider *myprv = (GdaMysqlProvider *) provider;
 
 	g_return_val_if_fail (GDA_IS_MYSQL_PROVIDER (myprv), FALSE);
@@ -318,9 +411,8 @@ gda_mysql_provider_create_database (GdaServerProvider *provider,
 	}
 
 	sql = g_strdup_printf ("CREATE DATABASE %s", name);
-	rc = mysql_real_query (mysql, sql, strlen (sql));
+	rc = mysql_query (mysql, sql);
 	g_free (sql);
-
 	if (rc != 0) {
 		gda_connection_add_error (cnc, gda_mysql_make_error (mysql));
 		return FALSE;
@@ -336,8 +428,8 @@ gda_mysql_provider_drop_database (GdaServerProvider *provider,
 				  const gchar *name)
 {
 	gint rc;
-	MYSQL *mysql;
 	gchar *sql;
+	MYSQL *mysql;
 	GdaMysqlProvider *myprv = (GdaMysqlProvider *) provider;
 
 	g_return_val_if_fail (GDA_IS_MYSQL_PROVIDER (myprv), FALSE);
@@ -350,10 +442,9 @@ gda_mysql_provider_drop_database (GdaServerProvider *provider,
 		return FALSE;
 	}
 
-	sql = g_strdup_printf ("DROP DATABASE IF EXISTS %s", name);
-	rc = mysql_real_query (mysql, sql, strlen (sql));
+	sql = g_strdup_printf ("DROP DATABASE %s", name);
+	rc = mysql_query (mysql, sql);
 	g_free (sql);
-
 	if (rc != 0) {
 		gda_connection_add_error (cnc, gda_mysql_make_error (mysql));
 		return FALSE;
@@ -403,7 +494,7 @@ gda_mysql_provider_execute_command (GdaServerProvider *provider,
 static gboolean
 gda_mysql_provider_begin_transaction (GdaServerProvider *provider,
 				      GdaConnection *cnc,
-				      const gchar *trans_id)
+				      GdaTransaction *xaction)
 {
 	MYSQL *mysql;
 	gint rc;
@@ -411,6 +502,7 @@ gda_mysql_provider_begin_transaction (GdaServerProvider *provider,
 
 	g_return_val_if_fail (GDA_IS_MYSQL_PROVIDER (myprv), FALSE);
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
+	g_return_val_if_fail (GDA_IS_TRANSACTION (xaction), FALSE);
 
 	mysql = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_MYSQL_HANDLE);
 	if (!mysql) {
@@ -418,6 +510,30 @@ gda_mysql_provider_begin_transaction (GdaServerProvider *provider,
 		return FALSE;
 	}
 
+	/* set isolation level */
+	switch (gda_transaction_get_isolation_level (xaction)) {
+	case GDA_TRANSACTION_ISOLATION_READ_COMMITTED :
+		rc = mysql_real_query (mysql, "SET TRANSACTION ISOLATION LEVEL READ COMMITTED", 46);
+		break;
+	case GDA_TRANSACTION_ISOLATION_READ_UNCOMMITTED :
+		rc = mysql_real_query (mysql, "SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED", 48);
+		break;
+	case GDA_TRANSACTION_ISOLATION_REPEATABLE_READ :
+		rc = mysql_real_query (mysql, "SET TRANSACTION ISOLATION LEVEL REPEATABLE READ", 47);
+		break;
+	case GDA_TRANSACTION_ISOLATION_SERIALIZABLE :
+		rc = mysql_real_query (mysql, "SET TRANSACTION ISOLATION LEVEL SERIALIZABLE", 44);
+		break;
+	default :
+		rc = 0;
+	}
+
+	if (rc != 0) {
+		gda_connection_add_error (cnc, gda_mysql_make_error (mysql));
+		return FALSE;
+	}
+
+	/* start the transaction */
 	rc = mysql_real_query (mysql, "BEGIN", strlen ("BEGIN"));
 	if (rc != 0) {
 		gda_connection_add_error (cnc, gda_mysql_make_error (mysql));
@@ -431,7 +547,7 @@ gda_mysql_provider_begin_transaction (GdaServerProvider *provider,
 static gboolean
 gda_mysql_provider_commit_transaction (GdaServerProvider *provider,
 				       GdaConnection *cnc,
-				       const gchar *trans_id)
+				       GdaTransaction *xaction)
 {
 	MYSQL *mysql;
 	gint rc;
@@ -459,7 +575,7 @@ gda_mysql_provider_commit_transaction (GdaServerProvider *provider,
 static gboolean
 gda_mysql_provider_rollback_transaction (GdaServerProvider *provider,
 					 GdaConnection *cnc,
-					 const gchar *trans_id)
+					 GdaTransaction *xaction)
 {
 	MYSQL *mysql;
 	gint rc;
@@ -505,7 +621,7 @@ gda_mysql_provider_supports (GdaServerProvider *provider,
 }
 
 static void
-add_string_row (GdaDataModelArray *recset, const gchar *str)
+add_aggregate_row (GdaDataModelArray *recset, const gchar *str)
 {
 	GdaValue *value;
 	GList list;
@@ -537,142 +653,142 @@ get_mysql_aggregates (GdaConnection *cnc, GdaParameterList *params)
 	//gda_server_recordset_model_set_field_gdatype (recset, 0, GDA_TYPE_STRING);
 
 	/* fill the recordset */
-	add_string_row (recset, "abs");
-	add_string_row (recset, "acos");
-	add_string_row (recset, "adddate");
-	add_string_row (recset, "ascii");
-	add_string_row (recset, "asin");
-	add_string_row (recset, "atan");
-	add_string_row (recset, "atan2");
-	add_string_row (recset, "benchmark");
-	add_string_row (recset, "bin");
-	add_string_row (recset, "bit_count");
-	add_string_row (recset, "ceiling");
-	add_string_row (recset, "char");
-	add_string_row (recset, "char_length");
-	add_string_row (recset, "character_length");
-	add_string_row (recset, "coalesce");
-	add_string_row (recset, "concat");
-	add_string_row (recset, "concat_ws");
-	add_string_row (recset, "connection_id");
-	add_string_row (recset, "conv");
-	add_string_row (recset, "cos");
-	add_string_row (recset, "cot");
-	add_string_row (recset, "count");
-	add_string_row (recset, "curdate");
-	add_string_row (recset, "current_date");
-	add_string_row (recset, "current_time");
-	add_string_row (recset, "current_timestamp");
-	add_string_row (recset, "curtime");
-	add_string_row (recset, "database");
-	add_string_row (recset, "date_add");
-	add_string_row (recset, "date_format");
-	add_string_row (recset, "date_sub");
-	add_string_row (recset, "dayname");
-	add_string_row (recset, "dayofmonth");
-	add_string_row (recset, "dayofweek");
-	add_string_row (recset, "dayofyear");
-	add_string_row (recset, "decode");
-	add_string_row (recset, "degrees");
-	add_string_row (recset, "elt");
-	add_string_row (recset, "encode");
-	add_string_row (recset, "encrypt");
-	add_string_row (recset, "exp");
-	add_string_row (recset, "export_set");
-	add_string_row (recset, "extract");
-	add_string_row (recset, "field");
-	add_string_row (recset, "find_in_set");
-	add_string_row (recset, "floor");
-	add_string_row (recset, "format");
-	add_string_row (recset, "from_days");
-	add_string_row (recset, "from_unixtime");
-	add_string_row (recset, "get_lock");
-	add_string_row (recset, "greatest");
-	add_string_row (recset, "hex");
-	add_string_row (recset, "hour");
-	add_string_row (recset, "if");
-	add_string_row (recset, "ifnull");
-	add_string_row (recset, "inet_aton");
-	add_string_row (recset, "inet_ntoa");
-	add_string_row (recset, "insert");
-	add_string_row (recset, "instr");
-	add_string_row (recset, "interval");
-	add_string_row (recset, "isnull");
-	add_string_row (recset, "last_insert_id");
-	add_string_row (recset, "lcase");
-	add_string_row (recset, "least");
-	add_string_row (recset, "left");
-	add_string_row (recset, "length");
-	add_string_row (recset, "load_file");
-	add_string_row (recset, "locate");
-	add_string_row (recset, "log");
-	add_string_row (recset, "log10");
-	add_string_row (recset, "lower");
-	add_string_row (recset, "lpad");
-	add_string_row (recset, "ltrim");
-	add_string_row (recset, "make_set");
-	add_string_row (recset, "master_pos_wait");
-	add_string_row (recset, "match");
-	add_string_row (recset, "max");
-	add_string_row (recset, "md5");
-	add_string_row (recset, "mid");
-	add_string_row (recset, "min");
-	add_string_row (recset, "minute");
-	add_string_row (recset, "mod");
-	add_string_row (recset, "month");
-	add_string_row (recset, "monthname");
-	add_string_row (recset, "now");
-	add_string_row (recset, "nullif");
-	add_string_row (recset, "oct");
-	add_string_row (recset, "octet_length");
-	add_string_row (recset, "ord");
-	add_string_row (recset, "password");
-	add_string_row (recset, "period_add");
-	add_string_row (recset, "period_diff");
-	add_string_row (recset, "pi");
-	add_string_row (recset, "position");	
-	add_string_row (recset, "pow");
-	add_string_row (recset, "power");
-	add_string_row (recset, "quarter");
-	add_string_row (recset, "radians");
-	add_string_row (recset, "rand");
-	add_string_row (recset, "release_lock");
-	add_string_row (recset, "repeat");
-	add_string_row (recset, "replace");	
-	add_string_row (recset, "reverse");
-	add_string_row (recset, "right");
-	add_string_row (recset, "round");
-	add_string_row (recset, "rpad");
-	add_string_row (recset, "rtrim");
-	add_string_row (recset, "second");
-	add_string_row (recset, "sec_to_time");
-	add_string_row (recset, "session_user");
-	add_string_row (recset, "sign");
-	add_string_row (recset, "sin");
-	add_string_row (recset, "soundex");
-	add_string_row (recset, "space");
-	add_string_row (recset, "sqrt");
-	add_string_row (recset, "strcmp");
-	add_string_row (recset, "subdate");
-	add_string_row (recset, "substring");
-	add_string_row (recset, "substring_index");
-	add_string_row (recset, "sysdate");
-	add_string_row (recset, "system_user");
-	add_string_row (recset, "tan");
-	add_string_row (recset, "time_format");
-	add_string_row (recset, "time_to_sec");
-	add_string_row (recset, "to_days");
-	add_string_row (recset, "trim");
-	add_string_row (recset, "truncate");
-	add_string_row (recset, "ucase");
-	add_string_row (recset, "unix_timestamp");
-	add_string_row (recset, "upper");
-	add_string_row (recset, "user");
-	add_string_row (recset, "version");
-	add_string_row (recset, "week");
-	add_string_row (recset, "weekday");
-	add_string_row (recset, "year");
-	add_string_row (recset, "yearweek");
+	add_aggregate_row (recset, "abs");
+	add_aggregate_row (recset, "acos");
+	add_aggregate_row (recset, "adddate");
+	add_aggregate_row (recset, "ascii");
+	add_aggregate_row (recset, "asin");
+	add_aggregate_row (recset, "atan");
+	add_aggregate_row (recset, "atan2");
+	add_aggregate_row (recset, "benchmark");
+	add_aggregate_row (recset, "bin");
+	add_aggregate_row (recset, "bit_count");
+	add_aggregate_row (recset, "ceiling");
+	add_aggregate_row (recset, "char");
+	add_aggregate_row (recset, "char_length");
+	add_aggregate_row (recset, "character_length");
+	add_aggregate_row (recset, "coalesce");
+	add_aggregate_row (recset, "concat");
+	add_aggregate_row (recset, "concat_ws");
+	add_aggregate_row (recset, "connection_id");
+	add_aggregate_row (recset, "conv");
+	add_aggregate_row (recset, "cos");
+	add_aggregate_row (recset, "cot");
+	add_aggregate_row (recset, "count");
+	add_aggregate_row (recset, "curdate");
+	add_aggregate_row (recset, "current_date");
+	add_aggregate_row (recset, "current_time");
+	add_aggregate_row (recset, "current_timestamp");
+	add_aggregate_row (recset, "curtime");
+	add_aggregate_row (recset, "database");
+	add_aggregate_row (recset, "date_add");
+	add_aggregate_row (recset, "date_format");
+	add_aggregate_row (recset, "date_sub");
+	add_aggregate_row (recset, "dayname");
+	add_aggregate_row (recset, "dayofmonth");
+	add_aggregate_row (recset, "dayofweek");
+	add_aggregate_row (recset, "dayofyear");
+	add_aggregate_row (recset, "decode");
+	add_aggregate_row (recset, "degrees");
+	add_aggregate_row (recset, "elt");
+	add_aggregate_row (recset, "encode");
+	add_aggregate_row (recset, "encrypt");
+	add_aggregate_row (recset, "exp");
+	add_aggregate_row (recset, "export_set");
+	add_aggregate_row (recset, "extract");
+	add_aggregate_row (recset, "field");
+	add_aggregate_row (recset, "find_in_set");
+	add_aggregate_row (recset, "floor");
+	add_aggregate_row (recset, "format");
+	add_aggregate_row (recset, "from_days");
+	add_aggregate_row (recset, "from_unixtime");
+	add_aggregate_row (recset, "get_lock");
+	add_aggregate_row (recset, "greatest");
+	add_aggregate_row (recset, "hex");
+	add_aggregate_row (recset, "hour");
+	add_aggregate_row (recset, "if");
+	add_aggregate_row (recset, "ifnull");
+	add_aggregate_row (recset, "inet_aton");
+	add_aggregate_row (recset, "inet_ntoa");
+	add_aggregate_row (recset, "insert");
+	add_aggregate_row (recset, "instr");
+	add_aggregate_row (recset, "interval");
+	add_aggregate_row (recset, "isnull");
+	add_aggregate_row (recset, "last_insert_id");
+	add_aggregate_row (recset, "lcase");
+	add_aggregate_row (recset, "least");
+	add_aggregate_row (recset, "left");
+	add_aggregate_row (recset, "length");
+	add_aggregate_row (recset, "load_file");
+	add_aggregate_row (recset, "locate");
+	add_aggregate_row (recset, "log");
+	add_aggregate_row (recset, "log10");
+	add_aggregate_row (recset, "lower");
+	add_aggregate_row (recset, "lpad");
+	add_aggregate_row (recset, "ltrim");
+	add_aggregate_row (recset, "make_set");
+	add_aggregate_row (recset, "master_pos_wait");
+	add_aggregate_row (recset, "match");
+	add_aggregate_row (recset, "max");
+	add_aggregate_row (recset, "md5");
+	add_aggregate_row (recset, "mid");
+	add_aggregate_row (recset, "min");
+	add_aggregate_row (recset, "minute");
+	add_aggregate_row (recset, "mod");
+	add_aggregate_row (recset, "month");
+	add_aggregate_row (recset, "monthname");
+	add_aggregate_row (recset, "now");
+	add_aggregate_row (recset, "nullif");
+	add_aggregate_row (recset, "oct");
+	add_aggregate_row (recset, "octet_length");
+	add_aggregate_row (recset, "ord");
+	add_aggregate_row (recset, "password");
+	add_aggregate_row (recset, "period_add");
+	add_aggregate_row (recset, "period_diff");
+	add_aggregate_row (recset, "pi");
+	add_aggregate_row (recset, "position");	
+	add_aggregate_row (recset, "pow");
+	add_aggregate_row (recset, "power");
+	add_aggregate_row (recset, "quarter");
+	add_aggregate_row (recset, "radians");
+	add_aggregate_row (recset, "rand");
+	add_aggregate_row (recset, "release_lock");
+	add_aggregate_row (recset, "repeat");
+	add_aggregate_row (recset, "replace");	
+	add_aggregate_row (recset, "reverse");
+	add_aggregate_row (recset, "right");
+	add_aggregate_row (recset, "round");
+	add_aggregate_row (recset, "rpad");
+	add_aggregate_row (recset, "rtrim");
+	add_aggregate_row (recset, "second");
+	add_aggregate_row (recset, "sec_to_time");
+	add_aggregate_row (recset, "session_user");
+	add_aggregate_row (recset, "sign");
+	add_aggregate_row (recset, "sin");
+	add_aggregate_row (recset, "soundex");
+	add_aggregate_row (recset, "space");
+	add_aggregate_row (recset, "sqrt");
+	add_aggregate_row (recset, "strcmp");
+	add_aggregate_row (recset, "subdate");
+	add_aggregate_row (recset, "substring");
+	add_aggregate_row (recset, "substring_index");
+	add_aggregate_row (recset, "sysdate");
+	add_aggregate_row (recset, "system_user");
+	add_aggregate_row (recset, "tan");
+	add_aggregate_row (recset, "time_format");
+	add_aggregate_row (recset, "time_to_sec");
+	add_aggregate_row (recset, "to_days");
+	add_aggregate_row (recset, "trim");
+	add_aggregate_row (recset, "truncate");
+	add_aggregate_row (recset, "ucase");
+	add_aggregate_row (recset, "unix_timestamp");
+	add_aggregate_row (recset, "upper");
+	add_aggregate_row (recset, "user");
+	add_aggregate_row (recset, "version");
+	add_aggregate_row (recset, "week");
+	add_aggregate_row (recset, "weekday");
+	add_aggregate_row (recset, "year");
+	add_aggregate_row (recset, "yearweek");
 
 	return GDA_DATA_MODEL (recset);
 }
@@ -700,51 +816,119 @@ get_mysql_tables (GdaConnection *cnc, GdaParameterList *params)
 {
 	GList *reclist;
 	GdaMysqlRecordset *recset;
+	GdaDataModelArray *model;
+	gint rows, r;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 
-	reclist = process_sql_commands (NULL, cnc, "show tables");
+	reclist = process_sql_commands (NULL, cnc, "show table status");
 	if (!reclist)
 		return NULL;
 
-	recset = GDA_MYSQL_RECORDSET (reclist->data);
+	recset = (GdaMysqlRecordset *) reclist->data;
 	g_list_free (reclist);
+	if (!GDA_IS_MYSQL_RECORDSET (recset))
+		return NULL;
 
-	return GDA_DATA_MODEL (recset);
+	/* add the extra information */
+	model = gda_data_model_array_new (4);
+	rows = gda_data_model_get_n_rows (GDA_DATA_MODEL (recset));
+	for (r = 0; r < rows; r++) {
+		GList *value_list = NULL;
+		gchar *name;
+		gchar *str;
+
+		/* 1st, the name of the table */
+		name = gda_value_stringify (gda_data_model_get_value_at (GDA_DATA_MODEL (recset), 0, r));
+		value_list = g_list_append (value_list, gda_value_new_string (name));
+
+		/* 2nd, the owner */
+		value_list = g_list_append (value_list, gda_value_new_string (""));
+
+		/* 3rd, the comments */
+		str = gda_value_stringify (gda_data_model_get_value_at (GDA_DATA_MODEL (recset), 14, r));
+		value_list = g_list_append (value_list, gda_value_new_string (str));
+		g_free (str);
+
+		/* 4th, the SQL command */
+		str = g_strdup_printf ("SHOW CREATE TABLE %s", name);
+		reclist = process_sql_commands (NULL, cnc, str);
+		g_free (str);
+		if (reclist && GDA_IS_DATA_MODEL (reclist->data)) {
+			str = gda_value_stringify (
+				gda_data_model_get_value_at (GDA_DATA_MODEL (reclist->data), 1, 0));
+			value_list = g_list_append (value_list, gda_value_new_string (str));
+
+			g_free (str);
+			g_list_foreach (reclist, (GFunc) g_object_unref, NULL);
+			g_list_free (reclist);
+		}
+		else
+			value_list = g_list_append (value_list, gda_value_new_string (""));
+
+		gda_data_model_append_row (GDA_DATA_MODEL (model), value_list);
+
+		g_free (name);
+		g_list_foreach (value_list, (GFunc) gda_value_free, NULL);
+		g_list_free (value_list);
+	}
+
+	return GDA_DATA_MODEL (model);
 }
 
 static GdaDataModel *
 get_mysql_types (GdaConnection *cnc, GdaParameterList *params)
 {
 	GdaDataModelArray *recset;
+	gint i;
+	struct {
+		const gchar *name;
+		const gchar *owner;
+		const gchar *comments;
+		GdaValueType type;
+	} types[] = {
+		{ "blob", "", "Binary blob", GDA_VALUE_TYPE_BINARY },
+		{ "date", "", "Date", GDA_VALUE_TYPE_DATE },
+		{ "datetime", "", "Date and time", GDA_VALUE_TYPE_TIMESTAMP },
+		{ "decimal", "", "Decimal number", GDA_VALUE_TYPE_DOUBLE },
+		{ "double", "", "Double precision number", GDA_VALUE_TYPE_DOUBLE },
+		{ "enum", "", "Enumeration", GDA_VALUE_TYPE_STRING },
+		{ "float", "", "Single precision number", GDA_VALUE_TYPE_SINGLE },
+		{ "int24", "", "24 bit integer", GDA_VALUE_TYPE_BIGINT },
+		{ "long", "", "Long integer", GDA_VALUE_TYPE_INTEGER },
+		{ "longlong", "", "Extra long integer", GDA_VALUE_TYPE_BIGINT },
+		{ "set", "", "Set", GDA_VALUE_TYPE_STRING },
+		{ "short", "", "Short integer", GDA_VALUE_TYPE_SMALLINT },
+		{ "string", "", "String", GDA_VALUE_TYPE_STRING },
+		{ "time", "", "Time", GDA_VALUE_TYPE_TIME },
+		{ "timestamp", "", "Time stamp", GDA_VALUE_TYPE_TIMESTAMP },
+		{ "tiny", "", "Tiny integer", GDA_VALUE_TYPE_SMALLINT },
+		{ "year", "", "Year", GDA_VALUE_TYPE_INTEGER }
+	};
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 
 	/* create the recordset */
-	recset = (GdaDataModelArray *) gda_data_model_array_new (1);
-	//gda_server_recordset_model_set_field_defined_size (recset, 0, 32);
+	recset = (GdaDataModelArray *) gda_data_model_array_new (4);
 	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 0, _("Type"));
-	//gda_server_recordset_model_set_field_scale (recset, 0, 0);
-	//gda_server_recordset_model_set_field_gdatype (recset, 0, GDA_TYPE_STRING);
+	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 1, _("Owner"));
+	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 2, _("Comments"));
+	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 3, _("GDA type"));
 
 	/* fill the recordset */
-	add_string_row (recset, "blob");
-	add_string_row (recset, "date");
-	add_string_row (recset, "datetime");
-	add_string_row (recset, "decimal");
-	add_string_row (recset, "double");
-	add_string_row (recset, "enum");
-	add_string_row (recset, "float");
-	add_string_row (recset, "int24");
-	add_string_row (recset, "long");
-	add_string_row (recset, "longlong");
-	add_string_row (recset, "set");
-	add_string_row (recset, "short");
-	add_string_row (recset, "string");
-	add_string_row (recset, "time");
-	add_string_row (recset, "timestamp");
-	add_string_row (recset, "tiny");
-	add_string_row (recset, "year");
+	for (i = 0; i < sizeof (types) / sizeof (types[0]); i++) {
+		GList *value_list = NULL;
+
+		value_list = g_list_append (value_list, gda_value_new_string (types[i].name));
+		value_list = g_list_append (value_list, gda_value_new_string (types[i].owner));
+		value_list = g_list_append (value_list, gda_value_new_string (types[i].comments));
+		value_list = g_list_append (value_list, gda_value_new_type (types[i].type));
+
+		gda_data_model_append_row (GDA_DATA_MODEL (recset), value_list);
+
+		g_list_foreach (value_list, (GFunc) gda_value_free, NULL);
+		g_list_free (value_list);
+	}
 
 	return GDA_DATA_MODEL (recset);
 }
@@ -803,6 +987,9 @@ field_row_to_value_list (MYSQL_ROW mysql_row)
 	/* references */
 	value_list = g_list_append (value_list, gda_value_new_string (mysql_row[5]));
 
+	/* default value */
+	value_list = g_list_append (value_list, gda_value_new_string (mysql_row[4]));
+
 	return value_list;
 }
 
@@ -820,7 +1007,7 @@ get_table_fields (GdaConnection *cnc, GdaParameterList *params)
 	struct {
 		const gchar *name;
 		GdaValueType type;
-	} fields_desc[8] = {
+	} fields_desc[] = {
 		{ N_("Field name")	, GDA_VALUE_TYPE_STRING  },
 		{ N_("Data type")	, GDA_VALUE_TYPE_STRING  },
 		{ N_("Size")		, GDA_VALUE_TYPE_INTEGER },
@@ -828,7 +1015,8 @@ get_table_fields (GdaConnection *cnc, GdaParameterList *params)
 		{ N_("Not null?")	, GDA_VALUE_TYPE_BOOLEAN },
 		{ N_("Primary key?")	, GDA_VALUE_TYPE_BOOLEAN },
 		{ N_("Unique index?")	, GDA_VALUE_TYPE_BOOLEAN },
-		{ N_("References")	, GDA_VALUE_TYPE_STRING  }
+		{ N_("References")	, GDA_VALUE_TYPE_STRING  },
+		{ N_("Default value")   , GDA_VALUE_TYPE_STRING  }
 	};
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
@@ -845,7 +1033,7 @@ get_table_fields (GdaConnection *cnc, GdaParameterList *params)
 	if (!par) {
 		gda_connection_add_error_string (
 			cnc,
-			_("You need to specify the name of the table"));
+			_("Table name is needed but none specified in parameter list"));
 		return NULL;
 	}
 
@@ -853,7 +1041,7 @@ get_table_fields (GdaConnection *cnc, GdaParameterList *params)
 	if (!table_name) {
 		gda_connection_add_error_string (
 			cnc,
-			_("You need to specify the name of the table"));
+			_("Table name is needed but none specified in parameter list"));
 		return NULL;
 	}
 
@@ -870,7 +1058,7 @@ get_table_fields (GdaConnection *cnc, GdaParameterList *params)
 	rows = mysql_num_rows (mysql_res);
 
 	/* fill in the recordset to be returned */
-	recset = (GdaDataModelArray *) gda_data_model_array_new (8);
+	recset = (GdaDataModelArray *) gda_data_model_array_new (9);
 	for (r = 0; r < sizeof (fields_desc) / sizeof (fields_desc[0]); r++) {
 		//gint defined_size =  (fields_desc[r].type == GDA_VALUE_TYPE_STRING) ? 64 : 
 		//	(fields_desc[r].type == GDA_VALUE_TYPE_INTEGER) ? sizeof (gint) : 1;
